#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
"""Install script for CMFCollection in an existing CMF/Plone installation.

Add the following external method to the root of your CMF instance:

  id:            installCMFCollection
  title:         Install CMFCollection (remove after install!)
  Module Name:   CMFCollection.Install
  Function Name: Install
"""
__version__ = "$Revision: $"

# $Source: $

#from os import path, listdir, stat ### FIX ME: delete unused
import os
import string
import re

from OFS.ObjectManager import BadRequestException

from App.Common import package_home
from Products.CMFCore.utils import getToolByName, minimalpath
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import DirectoryView, addDirectoryViews, createDirectoryView

from Products.CMFCollection import Names
from Products.CMFCollection.CollectionTool import CollectionTool
from Products.CMFCollection import CollectionType, collection_globals

def Install(self):
    """Register CMFCollection"""

    # setup install log
    out =  []
    write = out.append

    # get used CMF tools
    portal_url = getToolByName(self, 'portal_url')
    portal_skins = getToolByName(self, 'portal_skins')
    portal_catalog = getToolByName(self, 'portal_catalog')
    portal_types = getToolByName(self, 'portal_types')

    # get optional Plone tools
    portal_properties = getToolByName(self, 'portal_properties', None)

    # -----------------------------------------------------------------
    # setting up CMFCollection tool
    try:
        portal_url.getPortalObject()._setObject(CollectionTool.id, CollectionTool(CollectionTool.id))
        write("added 'portal_collections' to portal")
    except BadRequestException:
        write("could not add 'portal_collections' to portal (possibly because it already existed)")

    # -----------------------------------------------------------------
    # setting up catalog indexes and columns
    addIndex = portal_catalog.addIndex
    addColumn = portal_catalog.addColumn

    try:
        addIndex('puid', 'FieldIndex')
        write("added 'puid' to the catalog index")
    except:
        write("'puid' already added to the catalog index")
        
    try:
        addColumn('puid')
        write("added 'puid' as column to the catalog")
    except:
        write("'puid' already added as column to the catalog")

    try:
        portal_catalog.addIndex('originals_puid', 'FieldIndex')
        write("added 'originals_puid' to the catalog index")
    except:
        write("'originals_puid' already added to the catalog index")
        
    try:
        addColumn = portal_catalog.addColumn('originals_puid')
        write("added 'originals_puid' as column to the catalog")
    except:
        write("'originals_puid' already added as column to the catalog")

    # -----------------------------------------------------------------
    # setting up CMFCollection content type
    for t in CollectionType.factory_type_information:
        if t['id'] not in portal_types.objectIds():
            cfm = apply(ContentFactoryMetadata, (), t)
            portal_types._setObject(t['id'], cfm)
            write('Registered %s with the types tool' % t['id'])
        else:
            write('Object "%s" already existed in the types tool' % (t['id']))

    """ fix me: delete this!
    # add additional properties to factory type information 
    ctct = CollectionType.CollectionType
    fti = getattr(portal_types, ctct.meta_type)
    for prop in ctct._col_props:
        id = prop['id']
        if fti.hasProperty(id):
            write("'%s' already added to factory type information" % id)
        else:
            fti._properties = fti._properties + (prop,)
            fti.manage_changeProperties({id: getattr(ctct, "%s_default" % id)})
            write("Added '%s' to factory type information" % id)
    """

    # -----------------------------------------------------------------
    # Setup the skins
    ph = package_home(collection_globals)

    fp = minimalpath("%s/skins/collection" % ph)
    try:
        createDirectoryView(portal_skins, fp, 'collection')
        write("Added %s directory view to portal_skins" % fp)
    except:
        write("Couldn't add directory %s to portal_skins" % fp)

    fp = minimalpath("%s/skins/coll_ploneover" % ph)
    try:
        createDirectoryView(portal_skins, fp, 'coll_ploneover')
        write("Added %s directory view to portal_skins" % fp)
    except:
        write("Couldn't add directory %s to portal_skins" % fp)

    # Now we need to go through the skin configurations and insert
    # 'collection' into the configurations.  Preferably, this
    # should be right before where 'content' is placed.  Otherwise, we
    # append it to the end.
    skins = portal_skins.getSkinSelections()
    for skin in skins:
        path = portal_skins.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if 'coll_ploneover' not in path:
            try: path.insert(path.index('custom')+1, 'coll_ploneover')
            except ValueError:
                path.append('coll_ploneover')
            write("Added 'coll_ploneover' to %s skin" % skin)
        else:
            write("Skipping %s skin, 'coll_ploneover' is already set up" %
                  (skin))

        if 'collection' not in path:
            try: path.insert(path.index('content'), 'collection')
            except ValueError:
                path.append('collection')
            write("Added 'collection' to %s skin" % skin)
        else:
            write("Skipping %s skin, 'collection' is already set up" %
                  (skin))

        if 'collection/helpers' not in path:
            try: path.insert(path.index('collection'), 'collection/helpers')
            except ValueError:
                path.append('collection/helpers')
            write("Added 'collection/helpers' to %s skin" % skin)
        else:
            write("Skipping %s skin, 'collection/helpers' is already set up" %
                  (skin))

        path = string.join(path, ', ')
        # addSkinSelection will replace existing skins as well.
        portal_skins.addSkinSelection(skin, path)


    # -----------------------------------------------------------------
    # setup Plone properties

    res_pi = re.match('.*\.(.*?) at .*', str(portal_url.getPortalObject().__class__))
    isPloneSite = (res_pi.group(1) == 'PloneSite')

    if isPloneSite:
        write("\nAdd Plone navigation_properties:")
        props = (('default.collection_add.success', 'action:view')
                ,('default.collection_edit_form.success', 'script:collection_edit')
                ,('default.collection_edit.success', 'url:collection_contents')
                ,('default.collection_edit.failure', 'action:edit')
                ,('default.collection_update.success', 'url:collection_contents')
                ,('default.collection_move.success', 'url:collection_contents')
                ,('default.collection_down.success', 'url:collection_contents')
                ,('default.collection_down4.success', 'url:collection_contents')
                ,('default.collection_bottom.success', 'url:collection_contents')
                ,('default.collection_up.success', 'url:collection_contents')
                ,('default.collection_up4.success', 'url:collection_contents')
                ,('default.collection_top.success', 'url:collection_contents')
                )
        for p in props:
            if not portal_properties.navigation_properties.hasProperty(p[0]):
                portal_properties.navigation_properties.manage_addProperty(p[0], p[1], 'string')
                write("Added '%s' to 'navigation_properties'" % p[0])
            else:
                write("'%s' already added to 'navigation_properties' before" % p[0])
                
        write("\nAdd Plone form_properties:")
        props = (('collection_edit_form', 'validate_id,validate_collection_edit'),
                )
        for p in props:
            if not portal_properties.form_properties.hasProperty(p[0]):
                portal_properties.form_properties.manage_addProperty(p[0], p[1], 'string')
                write("Added '%s' to 'form_properties'" % p[0])
            else:
                write("'%s' already added to 'form_properties' before" % p[0])
                

    self.manage_permission(Names.ProposeContentToCollections,('Manager', 'Owner', 'Member', 'Reviewer'))
    self.manage_permission(Names.AddContentToCollections,('Manager','Owner'))
    write("Added permissions to portal root.")

    portal_actions = getToolByName(self, 'portal_actions')
    portal_actions.addActionProvider('portal_collections')
    write("Added permissions to portal root.")

    # return install log
    return '\n'.join(out)
