#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" Helper definitions

Definition and configuration of helpers beeing used by various collections.

$Id: $
"""
__version__ = "$Revision: $"

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder

from Products.CMFCore.utils import SimpleItemWithProperties
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.DCWorkflow.ContainerTab import ContainerTab

from Configuration import Configuration

class TemplateDefinition(SimpleItemWithProperties, Configuration):
    meta_type = 'Helper Template Reference'

    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'is_visible_info', 'type':'boolean', 'mode':'w'},
                   {'id':'template_id', 'type':'text', 'mode':'w'},
                  )

    def __init__(self, id):
        self.id = id

    title = ''
    is_visible_info = None
    template_id = ''

    security = ClassSecurityInfo()
    security.declareObjectProtected(ManagePortal)

InitializeClass(TemplateDefinition)

class HelperTemplates(ContainerTab, Configuration):
    meta_type = 'Collection Helper Templates'

    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'description', 'type':'text', 'mode':'w'},
                  )

    title = ''
    description = ''

    security = ClassSecurityInfo()
    security.declareObjectProtected(ManagePortal)

    all_meta_types = ({'name':TemplateDefinition.meta_type,
                       'action':'manage_addTemplateForm',
                       },)

    manage_options = Folder.manage_options

    manage_addTemplateForm = DTMLFile('zmi/addTemplateForm', globals())

    def _addObject(self, id):
        obj = TemplateDefinition(id)
        self._setObject(id, obj)
        return self._getOb(id)

    def manage_addTemplate(self, id, REQUEST=None):
        """Add a template
        """
        self._addObject(id)
        if REQUEST is not None:
            return self.manage_main(REQUEST, 'Helper added.')

InitializeClass(HelperTemplates)


class Helpers(ContainerTab, Configuration):
    meta_type = 'Collection Helpers'
    helpers_id = 'Helpers'

    def __init__(self, id):
        ContainerTab.__init__(self, id)

    security = ClassSecurityInfo()
    security.declareObjectProtected(ManagePortal)

    all_meta_types = ({'name':HelperTemplates.meta_type,
                       'action':'manage_addHelperForm',
                       },)

    manage_options = Folder.manage_options

    manage_addHelperForm = DTMLFile('zmi/addHelperForm', globals())

    def _addObject(self, id):
        obj = HelperTemplates(id)
        self._setObject(id, obj)
        return self._getOb(id)

    def manage_addHelper(self, id, REQUEST=None):
        """Add a helper
        """
        self._addObject(id)
        if REQUEST is not None:
            return self.manage_main(REQUEST, 'Helper added.')

InitializeClass(Helpers)
