#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
# The code below is derived from the original work of Maik Jablonski.
# The above copyright applies to the changed parts of the code only.
#########################################################################
####
##
# OrdererdObjectManager - maik.jablonski@uni-bielefeld.de
#
# This software uses modified codes from the OrderFolder-Product
# written by Stephan Richter / copyrighted by iuveno AG.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##
####
""" XXX



$Id: $
"""
__version__ = "$Revision: $"


from types import StringType
from Globals import ImageFile, DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions

from Products.CMFCore.PortalFolder import PortalFolder

oomPermission = 'Copy or Move'

class OrderedPortalFolder(PortalFolder):

    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)

    # override the ZMI folder view
    manage_main = DTMLFile('dtml/manage_main', globals())

    misc_ = {'up'     : ImageFile('zmi/up.gif', globals()),
             'down'   : ImageFile('zmi/down.gif', globals()),
             'top'    : ImageFile('zmi/top.gif', globals()),
             'bottom' : ImageFile('zmi/bottom.gif', globals())}

    security.declareProtected(oomPermission, 'get_object_position')
    def get_object_position(self, id):
        """ Get the position of a particular object. """
    
        ids = map(lambda obj: obj['id'], self._objects)
        if id in ids:
            return ids.index(id)
        else:
            raise 'ObjectNotFound', self.aq_parent._objects            
    
    security.declareProtected(oomPermission, 'move_objects_by_positions')
    def move_objects_by_positions(self, ids, position_delta):
        """ moves objects with ids by position_delta """

        if type(ids) is StringType: ids = (ids,)

        # Interestingly enough, we need to switch the order when
        # moving done, so that the movements won't cancel each other
        if position_delta > 0:
            ids = list(ids)
            ids.reverse()
            
        moved_objects = 0
        
        for id in ids:
            old_position = self.get_object_position(id)
            new_position = old_position + position_delta
            object = self._objects[old_position]
            # Make sure the new position makes sense and is valid
            if not (old_position == new_position  or
                    new_position > len(self._objects) or
                    new_position < 0):
            
                objects = list(self._objects)
                # now delete the entry ...
                objects = filter(lambda obj, id=id: obj['id'] != id, objects)
                # ... and now add it again
                objects.insert(new_position, object)
                self._objects = tuple(objects)

                moved_objects = moved_objects + 1

        return moved_objects

    security.declareProtected(oomPermission, 'move_objects_up')
    def move_objects_up(self, ids):
        """ moving the items up """
        
        return self.move_objects_by_positions(ids, -1)

    security.declareProtected(oomPermission, 'move_objects_down')
    def move_objects_down(self, ids):
        """ moving the items down """
 
        return self.move_objects_by_positions(ids, +1)

    security.declareProtected(oomPermission, 'move_objects_to_top')
    def move_objects_to_top(self, ids):
        """ moving the items to the top """
               
        if type(ids) is StringType: ids = (ids,)

        position_delta = - self.get_object_position(ids[0])
        return self.move_objects_by_positions(ids, position_delta)

    security.declareProtected(oomPermission, 'move_objects_to_bottom')
    def move_objects_to_bottom(self, ids):
        """ moving the items to the bottom """

        if type(ids) is StringType: ids = (ids,)

        # We will do the reverse twice,
        # but for going to the bottom there is no other choice.
        ids = list(ids)
        ids.reverse()

        position_delta = len(self.objectIds()) - self.get_object_position(ids[-1])
        return self.move_objects_by_positions(ids, position_delta)

    security.declareProtected(oomPermission, 'manage_move_objects_up')
    def manage_move_objects_up(self, ids=None, REQUEST=None):
        """ moves selected items up by one """

        if type(ids) is StringType: ids = (ids,)
            
        attempt = 0
        if ids: attempt = self.move_objects_up(ids)

        if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        return attempt

    security.declareProtected(oomPermission, 'manage_move_objects_down')
    def manage_move_objects_down(self, ids=None, REQUEST=None):
        """ moves selected items down by one """

        if type(ids) is StringType: ids = (ids,)

        attempt = 0
        if ids: attempt = self.move_objects_down(ids)

        if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        return attempt

    security.declareProtected(oomPermission, 'manage_move_objects_to_top')
    def manage_move_objects_to_top(self, ids=None, REQUEST=None):
        """ moves selected items to top """
    
        if type(ids) is StringType: ids = (ids,)

        attempt=0
        if ids: attempt=self.move_objects_to_top(ids)

        if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        return attempt

    security.declareProtected(oomPermission, 'manage_move_objects_to_bottom')
    def manage_move_objects_to_bottom(self, ids=None, REQUEST=None):
        """ moves selected items to bottom """

        if type(ids) is StringType: ids = (ids,)

        attempt=0
        if ids: attempt = self.move_objects_to_bottom(ids)

        if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        return attempt

    security.declareProtected(oomPermission, 'manage_orderObjects')
    def manage_orderObjects(self, ids=None, REQUEST=None):
        """ orders a container with the ordered list 'ids' """

        if ids:
            for id in ids:
                old_position = self.get_object_position(id)
                new_position = ids.index(id)
                object = self._objects[old_position]
                # Make sure the new position makes sense and is valid
                if not (old_position == new_position  or
                        new_position > len(self._objects) or
                        new_position < 0):
                
                    objects = list(self._objects)
                    # now delete the entry ...
                    objects = filter(lambda obj, id=id: obj['id'] != id, objects)
                    # ... and now add it again
                    objects.insert(new_position, object)
                    self._objects = tuple(objects)

        if REQUEST: REQUEST.RESPONSE.redirect('manage_main')
        
    # Override 'manage_renameObject' of CopyContainer -
    # manage_renameObject should preserve the order

    def manage_renameObject(self, id, new_id, REQUEST=None):
        """ rename a particular sub-object """
        position = self.get_object_position(id)
        PortalFolder.manage_renameObject(self, id, new_id, REQUEST)
        newposition = self.get_object_position(new_id)
        self.move_objects_by_positions(new_id, position - newposition)

        return new_id

InitializeClass(OrderedPortalFolder)
