#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" Slot definitions

Definition and configuration of slots beeing used by various collections.

$Id: $
"""
__version__ = "$Revision: $"

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder

from Products.CMFCore.utils import SimpleItemWithProperties
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.DCWorkflow.ContainerTab import ContainerTab

from Configuration import Configuration

class SlotDefinition(SimpleItemWithProperties, Configuration):
    meta_type = 'Collection Slot'

    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'description', 'type':'text', 'mode':'w'},
                   {'id':'visibility', 'type':'selection', 'mode':'w', 'select_variable':'visibilityTokens'},
                   {'id':'group', 'type':'selection', 'mode':'w', 'select_variable':'getAvailableGroups'},
                   {'id':'default_text', 'type':'text', 'mode':'w'},
                  )

    def __init__(self, id):
        self.id = id

    title = ''
    description = ''

    visibilityTokens = ['hidden', 'if not empty', 'always']
    visibility = visibilityTokens[1]

    def getAvailableGroups(self):
        return self.aq_parent.getProperty('groups')

    group = None

    default_text = ''

    security = ClassSecurityInfo()
    security.declareObjectProtected(ManagePortal)

InitializeClass(SlotDefinition)


class Slots(ContainerTab, Configuration):
    meta_type = 'Collection Slots'

    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'groups', 'type':'lines', 'mode':'w'}, )

    groups = []

    def __init__(self, id):
        ContainerTab.__init__(self, id)

    security = ClassSecurityInfo()
    security.declareObjectProtected(ManagePortal)

    all_meta_types = ({'name':SlotDefinition.meta_type,
                       'action':'manage_addSlotForm',
                       },)

    manage_options = Folder.manage_options

    manage_addSlotForm = DTMLFile('zmi/addSlotForm', globals())

    def _addObject(self, id):
        obj = SlotDefinition(id)
        self._setObject(id, obj)
        return self._getOb(id)

    def manage_addSlot(self, id, REQUEST=None):
        """Add a helper
        """
        self._addObject(id)
        if REQUEST is not None:
            return self.manage_main(REQUEST, 'Helper added.')

InitializeClass(Slots)
