#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" XXX



$Id: $
"""
__version__ = "$Revision: $"

import Zope
try:
    Zope.startup()
except NameError:  # pre Zope 2.6.1
    pass

from unittest import TestCase, TestSuite, makeSuite, main

from Acquisition import aq_base, aq_parent, aq_inner

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager

from Products.CMFCore.tests.base.dummy \
    import DummyObject, DummyFolder, DummyContent, addDummy, DummyFTI
from Products.CMFCore.tests.base.testcase import SecurityRequestTest
from Products.CMFCollection.tests.base.testcase \
    import SecurityTest
from Products.CMFCollection.tests.base.security \
    import UserWithRoles
#from Products.CMFCollection.tests.base.dummy import \
#     DummyContent, addDummy, DummyDocumentFTI

from Products.CMFCore import utils # we will monkey path this

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFDefault.URLTool import URLTool
from Products.CMFCore.TypesTool import TypesTool, FactoryTypeInformation
from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFDefault.URLTool import URLTool

from Products.CMFCollection import Names
from Products.CMFCollection import Types

from Products.CMFCollection.FactoryDefaults import \
     default_slots, default_helpers, default_types
from Products.CMFCollection.Types import Types
from Products.CMFCollection.CollectionTool import CollectionTool

from Products.CMFCollection.ObjectAnnotations import ObjectAnnotations
from Products.CMFCollection.CollectionType import addCollectionType
from Products.CMFCollection.CollectionSlot import addCollectionSlot

def set_checkPermission(method):
    """temp monkey path of CMFCore.utils.py _checkPermission.

    Returns the previous set method.
    """
    saved = utils._checkPermission
    utils._checkPermission = method
    return saved

fake_permisssion = None

def set_activePermission(permission):
    global fake_permission
    
    fake_permission = permission

def get_activePermission():
    global fake_permission
    
    return fake_permission

def my_checkPermission(permission, obj, StringType = type('')):
    global fake_permisssion
    if permission == fake_permission:
        return 1
    else:
        return 0

# Monkey Patch CMFDefault.Document.Document to avoid problems 
# during testing of copy & paste operations

saved_isMoveable = None
saved_isCopyable = None

def patchDummyDocument():
    global saved_isMoveable, saved_isCopyable
    from Products.CMFDefault.Document import Document
    if saved_isMoveable is None:
        saved_isMoveable = Document.cb_isMoveable
    if saved_isCopyable is None:
        saved_isCopyable = Document.cb_isCopyable
    Document.cb_isCopyable = Document.cb_isMoveable = lambda true=1:1

def unpatchDummyDocument():
    global saved_isMoveable, saved_isCopyable
    from Products.CMFDefault.Document import Document
    if saved_isMoveable is not None:
        Document.cb_isMoveable = saved_isMoveable
    if saved_isCopyable is not None:
        Document.cb_isCopyable = saved_isCopyable


class CollectionToolConfigurationTests(SecurityTest):
    # These tests ensure correct behaviour of the follwoing modules:
    #   - CollectionTool.py
    #   - Types.py
    #   - Slots.py
    #   - Helpers.py
    #   - Configuration.py mix in class
    #   - ConatinerTab.py implicitely (copied this module from DCWorkflow)

    def setUp(self):
        SecurityTest.setUp(self)
        self.root._setObject('portal_collections', CollectionTool(Names.tool_id))

    def test_configTree(self):
        # this test implies that instantiation of CollectionTool
        # instantiates at least:
        #   - one helper with at least one template
        #   - one slot
        #   - one type with at least one slot with at least on helper
        portal_collections = getattr(self.root, Names.tool_id)

        # test slot configuration
        slots = getattr(portal_collections, Names.slots_id)
        self._test_config(slots, default_slots, Names.slots_id)

        # test helper configuration
        helpers = getattr(portal_collections, Names.helpers_id)
        self._test_config(helpers, default_helpers, Names.helpers_id)

        # test type configuration
        types = getattr(portal_collections, Names.types_id)
        self._test_config(types, default_types, Names.types_id)

    def _test_config(self, obj, config, chain=''):
        for k,v in config.items():
            if k == '_properties':
                for pk,pv in v.items():
                    self.failUnless(obj.hasProperty(pk),
                        "property '%s' is missing in '%s'" % (pk, chain))
                    value = obj.getProperty(pk)
                    self.assertEqual(value, pv,
                        "property '%s' in '%s' has value '%s' instead of '%s'" % (pk, chain, value, pv))
            elif k == '_container':
                ids = v.keys()
                for o in obj.objectValues():
                    id = o.getId()
                    self.failUnless(id in ids,
                        "object '%s' missing in '%s'" % (id, chain))
                    self._test_config(o, v[id], '%s.%s' % (chain, id))

    def test_getTypesToolProperties(self):
        portal_collections = getattr(self.root, Names.tool_id)

        # get id and obj of a configured collection type
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        coll_obj = getattr(coll_config, coll_id)

        # add collection to types tool
        self.root._setObject('portal_types', TypesTool())
        portal_types = self.root.portal_types
        portal_types.manage_addTypeInformation(
            FactoryTypeInformation.meta_type,
            id=coll_id,
            typeinfo_name=Types._buildTypeInfoName(coll_config),
            )
        types_obj = getattr(portal_types, coll_id)

        # compare property values
        for k,v in types_obj.propertyItems():
            value = coll_obj.getProperty(k)
            self.assertEqual(value, v,
                "property '%s' in '%s' has value '%s' instead of '%s'" % (k, coll_id, value, v))
            
    def test_getAllowedTemplates(self):
        portal_collections = getattr(self.root, Names.tool_id)

        # get id and obj of a configured helper
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        coll_obj = getattr(coll_config, coll_id)
        slot_id = 'Main'
        slot_obj = getattr(coll_obj, slot_id)
        helper_id = default_types['_container'][coll_id]['_container'][slot_id]['_container'].keys()[0]
        helper_obj = getattr(slot_obj, helper_id)

        # get defined helpers of a slot
        coll_def = getattr(portal_collections, Names.helpers_id)
        helper_def = getattr(coll_def, helper_id)

        defined_templates = helper_def.objectIds()
        allowed_templates = helper_obj.getAllowedTemplates()

        for id in allowed_templates:
            self.failUnless(id in defined_templates)
        for id in defined_templates:
            self.failUnless(id in allowed_templates)

    ### fix me: yet untested: else part in Types._addObject()

class CollectionToolTests(SecurityRequestTest):

    def setUp(self):
        SecurityRequestTest.setUp(self)
        self.setUpToolInfrastructure()

    def setUpToolInfrastructure(self):
        # setup portal_url tool
        self.root._setObject('portal_url', URLTool())

        # setup portal_catalog tool
        self.root._setObject('portal_catalog', CatalogTool())
        portal_catalog = self.root.portal_catalog
        portal_catalog.addIndex('puid', 'FieldIndex')
        portal_catalog.addColumn('puid')

        # setup portal_collections tool with factory default configuration
        self.root._setObject(Names.tool_id, CollectionTool(Names.tool_id))
        portal_collections = getattr(self.root, Names.tool_id)

        # setup portal_types tool
        self.root._setObject('portal_types', TypesTool())
        portal_types = self.root.portal_types
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        portal_types.manage_addTypeInformation(
            FactoryTypeInformation.meta_type,
            id=coll_id,
            typeinfo_name=Types._buildTypeInfoName(coll_config),
            )
        portal_types._setObject('Dummy Content', DummyFTI)

    def _addCollection(self, container, id):
        # add a collection of type coll_id
        coll_id = default_types['_container'].keys()[0]
        obj_ref = container.invokeFactory(coll_id, id)
        # CollectionTypes invokeFactory returns added object reference
        # whereas other types don't
        if obj_ref is None:
            obj_ref = getattr(container, id)
        return obj_ref

    def _addFolder(self, container, id):
        container._setObject(id, PortalFolder(id, ''))
        obj_ref = getattr(container, id)
        return obj_ref

    def test_getSiteMap(self):
        set_checkPermission(my_checkPermission)

        # build a tree of collections and folders
        root = self._addFolder(self.root, 'root_folder')

        ci = self._addCollection(root, 'index_html')
        
        ci_c1 = self._addCollection(root, 'ci_c1')
        ci_c1_c1 = self._addCollection(ci_c1, 'ci_c1_c1')
        
        ci_c2 = self._addCollection(root, 'ci_c2')
        
        ci_f1 = self._addFolder(root, 'ci_f1')
        ci_f1_ci = self._addCollection(ci_f1, 'index_html')
        ci_f1_c1 = self._addCollection(ci_f1, 'ci_f1_c1')
        ci_f1_f1 = self._addFolder(ci_f1, 'ci_f1_f1')
        ci_f1_f1_f1 = self._addFolder(ci_f1_f1, 'ci_f1_f1_f1')
        ci_f1_f1_f1_f1 = self._addFolder(ci_f1_f1_f1, 'ci_f1_f1_f1_f1')
        ci_f1_f1_f1_f1_c1 = self._addCollection(ci_f1_f1_f1_f1, 'ci_f1_f1_f1_f1_c1')
        
        ci_c3 = self._addCollection(ci, 'ci_c3')
        ci_c3_c1 = self._addCollection(ci_c3, 'ci_c3_c1')
        
        ci_f2 = self._addFolder(root, 'ci_f2')
        ci_f2_c1 = self._addCollection(ci_f2, 'ci_f2_c1')
        
        maxdepth_default = 2
        result_default = [
            ('1 /root_folder/index_html'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1   /root_folder/index_html/Main/ci_c3/Main/ci_c3_c1'),
            ('1  /root_folder/ci_c1'),
            ('1   /root_folder/ci_c1/Main/ci_c1_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1/index_html'),
            ('1   /root_folder/ci_f1/ci_f1_c1'),
        ]
        maxdepth_base = 2
        result_base = [
            ('1 /root_folder'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1   /root_folder/index_html/Main/ci_c3/Main/ci_c3_c1'),
            ('1  /root_folder/ci_c1'),
            ('1   /root_folder/ci_c1/Main/ci_c1_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1'),
            ('1   /root_folder/ci_f1/ci_f1_c1'),
        ]
        maxdepth_extended = 5
        result_extended = [
            ('1 /root_folder/index_html'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1   /root_folder/index_html/Main/ci_c3/Main/ci_c3_c1'),
            ('1  /root_folder/ci_c1'),
            ('1   /root_folder/ci_c1/Main/ci_c1_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1/index_html'),
            ('1   /root_folder/ci_f1/ci_f1_c1'),
            ('0   /root_folder/ci_f1/ci_f1_f1'),
            ('0    /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1'),
            ('0     /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1/ci_f1_f1_f1_f1'),
            ('1      /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1/ci_f1_f1_f1_f1/ci_f1_f1_f1_f1_c1'),
            ('0  /root_folder/ci_f2'),
            ('1   /root_folder/ci_f2/ci_f2_c1'),
        ]
        maxdepth_extended_base = 5
        result_extended_base = [
            ('1 /root_folder'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1   /root_folder/index_html/Main/ci_c3/Main/ci_c3_c1'),
            ('1  /root_folder/ci_c1'),
            ('1   /root_folder/ci_c1/Main/ci_c1_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1'),
            ('1   /root_folder/ci_f1/ci_f1_c1'),
            ('0   /root_folder/ci_f1/ci_f1_f1'),
            ('0    /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1'),
            ('0     /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1/ci_f1_f1_f1_f1'),
            ('1      /root_folder/ci_f1/ci_f1_f1/ci_f1_f1_f1/ci_f1_f1_f1_f1/ci_f1_f1_f1_f1_c1'),
            ('0  /root_folder/ci_f2'),
            ('1   /root_folder/ci_f2/ci_f2_c1'),
        ]
        set_maxdepth_base_max = 1
        maxdepth_base_max = 1
        result_base_max = [
            ('1 /root_folder'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1  /root_folder/ci_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1'),
        ]
        set_maxdepth_extended_base_max = 4
        maxdepth_extended_base_max = 2
        result_extended_base_max = [
            ('1 /root_folder'),
            ('1  /root_folder/index_html/Main/ci_c3'),
            ('1   /root_folder/index_html/Main/ci_c3/Main/ci_c3_c1'),
            ('1  /root_folder/ci_c1'),
            ('1   /root_folder/ci_c1/Main/ci_c1_c1'),
            ('1  /root_folder/ci_c2'),
            ('1  /root_folder/ci_f1'),
            ('1   /root_folder/ci_f1/ci_f1_c1'),
            ('0  /root_folder/ci_f2'),
            ('1   /root_folder/ci_f2/ci_f2_c1'),
        ]
        portal_collections = getattr(self.root, Names.tool_id)
        root_pos = len(self.root.absolute_url())
        # test standard behaviour
        site_map = portal_collections.getSiteMap(root)
        self.assertEqual(site_map[1], maxdepth_default)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_default.pop(0))

        # test with BASE_REFERENCES option
        site_map = portal_collections.getSiteMap(root, 
            mode=portal_collections.BASE_REFERENCES)
        self.assertEqual(site_map[1], maxdepth_base)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_base.pop(0))

        # test with EXTENDED_SITEMAP option
        site_map = portal_collections.getSiteMap(root, 
            mode=portal_collections.EXTENDED_SITEMAP)
        self.assertEqual(site_map[1], maxdepth_extended)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_extended.pop(0))

        # test with BASE_REFERENCES and EXTENDED_SITEMAP option
        site_map = portal_collections.getSiteMap(root, 
            mode=portal_collections.BASE_REFERENCES | portal_collections.EXTENDED_SITEMAP)
        self.assertEqual(site_map[1], maxdepth_extended_base)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_extended_base.pop(0))

        # test with BASE_REFERENCES option and max depth
        site_map = portal_collections.getSiteMap(root, 
            maxdepth = set_maxdepth_base_max,
            mode=portal_collections.BASE_REFERENCES)
        self.assertEqual(site_map[1], maxdepth_base_max)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_base_max.pop(0))

        # test with BASE_REFERENCES and EXTENDED_SITEMAP option and max depth
        site_map = portal_collections.getSiteMap(root, 
            maxdepth = set_maxdepth_extended_base_max,
            mode=portal_collections.BASE_REFERENCES | portal_collections.EXTENDED_SITEMAP)
        self.assertEqual(site_map[1], maxdepth_extended_base_max)
        for node in site_map[0]:
            node_str = '%s %s%s' % (node[2], node[0]*' ', node[1].absolute_url()[root_pos:])
            self.assertEqual(node_str, result_extended_base_max.pop(0))


class CollectionPuidToolTests(SecurityTest):

    def setUp(self):
        SecurityTest.setUp(self)
        self.setUpToolInfrastructure()
        self.setUpContentInfrastructure()

    def setUpToolInfrastructure(self):
        # setup portal_catalog tool
        self.root._setObject('portal_catalog', CatalogTool())
        portal_catalog = self.root.portal_catalog
        portal_catalog.addIndex('puid', 'FieldIndex')
        portal_catalog.addColumn('puid')

        # setup portal_collections tool with factory default configuration
        self.root._setObject(Names.tool_id, CollectionTool(Names.tool_id))
        portal_collections = getattr(self.root, Names.tool_id)

        # setup portal_types tool
        self.root._setObject('portal_types', TypesTool())
        portal_types = self.root.portal_types
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        portal_types.manage_addTypeInformation(
            FactoryTypeInformation.meta_type,
            id=coll_id,
            typeinfo_name=Types._buildTypeInfoName(coll_config),
            )
        portal_types._setObject('Dummy Content', DummyFTI)

    def setUpContentInfrastructure(self):
        # set up portal folder
        self.root._setObject('portal', PortalFolder('portal', ''))

        # add dummy content
        self.root.portal.invokeFactory('Dummy Content', 'dummy1')
        self.root.portal.invokeFactory('Dummy Content', 'dummy2')

        noSecurityManager()
        newSecurityManager(None, UserWithRoles('Foo').__of__(self.root.portal))

    def test_noPuid(self):
        dummy1 =self.root.portal.dummy1
        portal_collections = getattr(self.root, Names.tool_id)

        self.assertEqual(portal_collections.PUID(dummy1), None)

    def test_getPuid(self):
        dummy1 =self.root.portal.dummy1
        dummy2 =self.root.portal.dummy2
        portal_collections = getattr(self.root, Names.tool_id)

        puid1 = portal_collections.getPuid(dummy1)
        puid2 = portal_collections.getPuid(dummy2)
        impossible = portal_collections.getImpossiblePuid()

        self.failIfEqual(puid1, impossible)
        self.failIfEqual(puid2, impossible)
        self.failIfEqual(puid1, puid2)

    def test_getObject(self):
        dummy1 =self.root.portal.dummy1
        dummy2 =self.root.portal.dummy2
        portal_collections = getattr(self.root, Names.tool_id)

        puid1 = portal_collections.getPuid(dummy1)
        puid2 = portal_collections.getPuid(dummy2)
        impossible = portal_collections.getImpossiblePuid()

        obj1 = portal_collections.getObject(puid1)
        obj2 = portal_collections.getObject(puid2)
        obj3 = portal_collections.getObject(impossible)

        self.assertEqual(obj1, dummy1)
        self.assertEqual(obj2, dummy2)
        self.failIfEqual(obj1, dummy2)
        self.assertEqual(obj3, None)

    def test_delPuid(self):
        dummy1 =self.root.portal.dummy1
        portal_collections = getattr(self.root, Names.tool_id)

        puid1 = portal_collections.getPuid(dummy1)
        impossible = portal_collections.getImpossiblePuid()

        portal_collections.delPuid(dummy1)

        puid1_del = portal_collections.PUID(dummy1)
        obj1 = portal_collections.getObject(puid1)

        self.assertEqual(puid1_del, impossible)
        self.assertEqual(obj1, None)


class ObjectAnnotationTests(TestCase):

    def test_defaults(self):
        anno = ObjectAnnotations()

        self.failIf(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failIf(anno.isBodyVisible())

        self.assertEqual(anno.getStatus(), Names.JustCreated)
        self.assertEqual(anno.getHelpers(), None)

    def test_setAndGetStatus(self):
        # test one path through the whole matrix only to see if the
        # transition mechanism works
        anno = ObjectAnnotations()
        anno.acceptStatus()
        self.assertEqual(anno.getStatus(), Names.UpToDate)
        anno.setStatus(Names.UpdateRequestPending)
        self.assertEqual(anno.getStatus(), Names.UpdateRequestPending)
        anno.setStatus(Names.Updated)
        self.assertEqual(anno.getStatus(), Names.Updated)
        anno.setStatus(Names.ChangeNotificationPending)
        self.assertEqual(anno.getStatus(), Names.ChangeNotificationPending)
        anno.rejectStatus()
        self.assertEqual(anno.getStatus(), Names.OutOfDate)

        # test impossible transitions
        anno = ObjectAnnotations()
        anno.setStatus(Names.AddRequestPending)
        anno.setStatus(Names.Added)
        self.assertEqual(anno.getStatus(), Names.AddRequestPending)

    def test_setGetVisibility(self):
        anno = ObjectAnnotations()

        anno.setVisibility(ObjectAnnotations.TITLE | ObjectAnnotations.BODY)
        self.failUnless(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failUnless(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.ALL)
        anno.setTitleVisibility(0)
        self.failIf(anno.isTitleVisible())
        self.failUnless(anno.isDescriptionVisible())
        self.failUnless(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.ALL)
        anno.setDescriptionVisibility(0)
        self.failUnless(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failUnless(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.ALL)
        anno.setBodyVisibility(0)
        self.failUnless(anno.isTitleVisible())
        self.failUnless(anno.isDescriptionVisible())
        self.failIf(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.NONE)
        anno.setTitleVisibility(1)
        self.failUnless(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failIf(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.NONE)
        anno.setDescriptionVisibility(1)
        self.failIf(anno.isTitleVisible())
        self.failUnless(anno.isDescriptionVisible())
        self.failIf(anno.isBodyVisible())

        anno.setVisibility(ObjectAnnotations.NONE)
        anno.setBodyVisibility(1)
        self.failIf(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failUnless(anno.isBodyVisible())

    def test_setGetDelHelperTemplates(self):
        anno = ObjectAnnotations()

        self.assertRaises(KeyError, anno.getTemplate, 'footer')
        self.assertRaises(KeyError, anno.getTemplate, 'spacer')

        anno.setTemplate('footer', 'foo')
        anno.setTemplate('spacer', 'bar')

        self.assertRaises(KeyError, anno.getTemplate, 'invalid')
        self.assertEqual(anno.getTemplate('footer'), 'foo')
        self.assertEqual(anno.getTemplate('spacer'), 'bar')

        anno.delHelper('footer')
        self.assertRaises(KeyError, anno.getTemplate, 'footer')
        self.assertEqual(anno.getTemplate('spacer'), 'bar')

        helpers = anno.getHelpers()

        self.failIf(helpers.get('footer'))
        self.assertEqual(helpers['spacer'], 'bar')
        
    def test_getSetAnnotation(self):
        anno = ObjectAnnotations()

        saved = anno.getAnnotation()
        
        anno.acceptStatus()
        anno.setVisibility(ObjectAnnotations.DESCRIPTION | ObjectAnnotations.BODY)
        anno.setHelpers('gaga')

        self.assertEqual(anno.getStatus(), Names.UpToDate)
        self.failIf(anno.isTitleVisible())
        self.failUnless(anno.isDescriptionVisible())
        self.failUnless(anno.isBodyVisible())
        self.assertEqual(anno.getHelpers(), 'gaga')

        anno.setAnnotation(saved)

        self.failIf(anno.isTitleVisible())
        self.failIf(anno.isDescriptionVisible())
        self.failIf(anno.isBodyVisible())

        self.assertEqual(anno.getStatus(), Names.JustCreated)
        self.assertEqual(anno.getHelpers(), None)

class CollectionSlotTests(SecurityTest):

    def setUp(self):
        SecurityTest.setUp(self)
        self.setUpToolInfrastructure()
        self.setUpContentInfrastructure()

    def setUpToolInfrastructure(self):
        # setup portal_catalog tool
        self.root._setObject('portal_catalog', CatalogTool())
        portal_catalog = self.root.portal_catalog
        portal_catalog.addIndex('puid', 'FieldIndex')
        portal_catalog.addColumn('puid')

        # setup portal_collections tool with factory default configuration
        self.root._setObject(Names.tool_id, CollectionTool(Names.tool_id))
        portal_collections = getattr(self.root, Names.tool_id)

        # setup portal_types tool
        self.root._setObject('portal_types', TypesTool())
        portal_types = self.root.portal_types
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        portal_types.manage_addTypeInformation(
            FactoryTypeInformation.meta_type,
            id=coll_id,
            typeinfo_name=Types._buildTypeInfoName(coll_config),
            )
        portal_types._setObject('Dummy Content', DummyFTI) 
        portal_types._setObject('Dummy Folder', DummyFTI) 

    def setUpContentInfrastructure(self):
        # set up portal folder
        self.root._setObject('portal', PortalFolder('portal', ''))
        self.root.portal._setObject('folder', PortalFolder('folder', ''))

        # add dummy content
        self.root.portal.invokeFactory('Dummy Content', 'dummy1')
        self.root.portal.invokeFactory('Dummy Content', 'dummy2')
        self.root.portal.invokeFactory('Dummy Content', 'dummy3')
        self.root.portal.invokeFactory('Dummy Content', 'dummy4')
        self.root.portal.invokeFactory('Dummy Content', 'dummy5')
        self.root.portal.folder.invokeFactory('Dummy Content', 'dummy2')
        setattr(self.root.portal.dummy1, 'gaga', 0)

        # setup factory defaults as configuration
        self.setUpConfigurationIdsAndProperties()

        # set up another portal folder acting as slot container
        # 'getConfigurationObject' needs a correct portal_type to
        # get to the slots configuration
        self.root._setObject('collection', PortalFolder('collection', ''))
        setattr(self.root.portal.collection, 'portal_type', self.coll_id)
        addCollectionSlot(self.root.portal.collection, self.slot_id)


    def setUpConfigurationIdsAndProperties(self):
        # ids from FactoryDefaults.py
        self.coll_id = coll_id = 'Collection'
        self.slot_id = slot_id = 'Main'
        self.helper_id = helper_id = 'Footers'

        default_types_coll = default_types['_container'][coll_id]
        self.coll_cfg_props = default_types_coll['_properties']

        default_types_slot = default_types_coll['_container'][slot_id]
        self.slot_cfg_props = default_types_slot['_properties']

        self.default_types_helpers = default_types_helpers = \
            default_types_slot['_container']
        default_types_helper = default_types_helpers[helper_id]
        self.helper_cfg_props = default_types_helper['_properties']

        default_slots_slot = default_slots['_container'][slot_id]
        self.slot_def_props = default_slots_slot['_properties']
        
    # fix me: unfortunately putting more than one 'test_*'
    # method causes errors in 'setUpContentInfrastructure'.
    # That's why I've put all tests into one test.

    def test_all(self):
        slot_obj = getattr(self.root.portal.collection, self.slot_id)

        dummy1 = self.root.portal.dummy1
        dummy2 = self.root.portal.dummy2
        dummy3 = self.root.portal.dummy3 # do not add to slot
        dummy4 = self.root.portal.dummy4
        dummy5 = self.root.portal.dummy5
        dummy22 = self.root.portal.folder.dummy2
        portal_collections = getattr(self.root, Names.tool_id)

        # positions depend on the 'initial_position' slot property
        initial_position = self.slot_cfg_props['initial_position']
        dummy1_pos = initial_position=='top' and 1 or 0
        dummy2_pos = initial_position!='top' and 1 or 0

        # (1) add objects and test if they got added and position,
        #     status, puids, originals puids, visibility and helper
        #     templates are set correctly
        self.failIf(getattr(aq_base(slot_obj), 'dummy1', None))
        self.failIf(getattr(aq_base(slot_obj), 'dummy2', None))
        slot_obj.addOrUpdateObject('add', dummy1, None, Names.UpToDate)
        slot_obj.addOrUpdateObject('add', dummy2, None, Names.DeleteNotificationPending)
        dummy1_inslot = getattr(aq_base(slot_obj), 'dummy1', None)
        dummy2_inslot = getattr(aq_base(slot_obj), 'dummy2', None)

        # test if object got added to slot
        self.failUnless(dummy1_inslot)
        self.failUnless(dummy2_inslot)

        # test if object has correct position
        self.assertEqual(slot_obj.getPos(dummy1_inslot), dummy1_pos)
        self.assertEqual(slot_obj.getPos(dummy2_inslot), dummy2_pos)

        # test status
        self.assertEqual(slot_obj.getStatus(dummy1_inslot), Names.UpToDate)
        self.assertEqual(slot_obj.getStatus(dummy2_inslot), Names.DeleteNotificationPending)

        # test original_puid
        self.assertEqual(dummy1_inslot.originals_puid, portal_collections.getPuid(dummy1))
        self.assertEqual(dummy2_inslot.originals_puid, portal_collections.getPuid(dummy2))

        # test if dummy object have a puid different of the original ones
        forbidden_puids = (portal_collections.PUID(dummy1),
                           portal_collections.PUID(dummy2), 
                           portal_collections.PUID(dummy22)) 
        self.failIf(portal_collections.PUID(dummy1_inslot) in forbidden_puids)
        self.failIf(portal_collections.PUID(dummy2_inslot) in forbidden_puids)

        # test getCopy method
        self.assertEqual(slot_obj.getCopy(dummy1), dummy1_inslot)
        self.assertEqual(slot_obj.getCopy(dummy3), None)

        # test if visibility is as set in the configuration
        self.assertEqual(not slot_obj.isTitleVisible(dummy1_inslot),
            not self.slot_cfg_props['default_title_visibility'])
        self.assertEqual(not slot_obj.isDescriptionVisible(dummy1_inslot),
            not self.slot_cfg_props['default_description_visibility'])
        self.assertEqual(not slot_obj.isBodyVisible(dummy1_inslot),
            not self.slot_cfg_props['default_body_visibility'])

        # test if helper template is as set in the configuration
        self.assertEqual(
            slot_obj.getSelectedTemplateId(dummy1_inslot, self.helper_id),
            self.helper_cfg_props['default_template'])
        for template in slot_obj.getAvailableTemplates(self.helper_id):
            self.failUnless(template['id'] in self.helper_cfg_props['allowed_templates'])
        for template_id in self.helper_cfg_props['allowed_templates']:
            self.failUnless(template_id in [t['id'] for t in slot_obj.getAvailableTemplates(self.helper_id)])

        # test helper listing
        helpers_test = map(lambda k:k.getId(), slot_obj.getHelperValues())
        helpers_cfg = self.default_types_helpers.keys()
        for helper_id in helpers_test:
            self.failUnless(helper_id in helpers_cfg)
        for helper_id in helpers_cfg:
            self.failUnless(helper_id in helpers_test)

        # test helper listing with choosable templates only
        helpers_test = map(lambda k:k.getId(),
            slot_obj.getHelperValues(choosable_only=1))
        dth = self.default_types_helpers
        helpers_cfg = filter(lambda h,dth=dth:
            len(dth[h]['_properties']['allowed_templates'])>1,
            self.default_types_helpers.keys())
        for helper_id in helpers_test:
            self.failUnless(helper_id in helpers_cfg)
        for helper_id in helpers_cfg:
            self.failUnless(helper_id in helpers_test)

        # get linked object depending on link policy
        newSecurityManager(None, UserWithRoles('Foo').__of__(self.root.portal))
        config = slot_obj._getConfigurationObject()
        dummy1_status = dummy1_inslot.collection.status
        dummy1_modification_date = dummy1.modification_date

        config._updateProperty('link_policy', Names.CopyAlways)
        dummy1_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1_inslot)
        dummy1_inslot.collection.status = Names.AddRequestPending
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1_inslot)

        config._updateProperty('link_policy', Names.CopyIfIdenticalOriginalElse)
        dummy1_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1_inslot)
#       XXX there is a problem in the test, implementation has changed
#        dummy1_inslot.collection.status = Names.AddRequestPending
#        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1)

        config._updateProperty('link_policy', Names.OriginalAlways)
        dummy1_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1)
        dummy1_inslot.collection.status = Names.AddRequestPending
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1)

        config._updateProperty('link_policy', Names.OriginalIfIdenticalCopyElse)
        dummy1_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1)
#       XXX there is a problem in the test, implementation has changed
#        dummy1_inslot.collection.status = Names.AddRequestPending
#        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1_inslot)

        config._updateProperty('track_changes', 1)
        config._updateProperty('auto_update', 0)
        config._updateProperty('link_policy', Names.CopyAlways)
        dummy1.modification_date += 1
        dummy1_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1_inslot)
        self.assertEqual(slot_obj.getStatus(dummy1_inslot), Names.ChangeNotificationPending)

        config._updateProperty('track_changes', 1)
        config._updateProperty('auto_update', 1)
        config._updateProperty('link_policy', Names.OriginalAlways)
        dummy1.modification_date += 1
        dummy1.gaga = 1.1
        dummy1_inslot.collection.status = Names.UpToDate
        self.failIfEqual(getattr(aq_base(dummy1_inslot), 'gaga', None), 1.1)
        noSecurityManager()
        self.assertEqual(slot_obj.getLinkedObject(dummy1_inslot), dummy1)
        newSecurityManager(None, UserWithRoles('Foo').__of__(self.root.portal))
        dummy1_inslot = getattr(aq_base(slot_obj), 'dummy1', None)
#       XXX there is a problem in the test, implementation has changed
#        self.assertEqual(slot_obj.getStatus(dummy1_inslot), Names.Updated)
#        self.assertEqual(dummy1_inslot.gaga, 1.1)

        noSecurityManager()
        slot_obj.addOrUpdateObject('add', dummy4, None, Names.UpToDate)
        dummy4_inslot = getattr(aq_base(slot_obj), 'dummy4', None)
        newSecurityManager(None, UserWithRoles('Foo').__of__(self.root.portal))
        self.root.portal.manage_delObjects('dummy4')
        config._updateProperty('track_changes', 1)
        config._updateProperty('link_policy', Names.CopyAlways)
        dummy4_inslot.collection.status = Names.UpToDate
        self.assertEqual(slot_obj.getLinkedObject(dummy4_inslot), dummy4_inslot)
        self.assertEqual(slot_obj.getStatus(dummy4_inslot), Names.DeleteNotificationPending)

        dummy1.modification_date = dummy1_modification_date
        dummy1_inslot.collection.status = dummy1_status
        noSecurityManager()

        # test helper rendering
        template_id = self.helper_cfg_props['default_template']
        template_meths = {}
        for hk, hv in default_helpers['_container'].items():
            for tk, tv in hv['_container'].items():
                if tk == template_id:
                    text = 'correct template rendered'
                else:
                    text = 'wrong template rendered'

                # attach python methods to slot to simulate configured
                # skins
                setattr(slot_obj, tv['_properties']['template_id'],
                    lambda container, obj, text=text: text)

        self.assertEqual(
            slot_obj.getRenderedHelper(dummy1_inslot, helper_ids=self.helper_id),
            'correct template rendered')

        cfg_helpers = self.default_types_helpers.keys()
        rendered_helpers = slot_obj.getRenderedHelper(dummy1_inslot)
        for h,r in rendered_helpers.items():
            self.failUnless(h in cfg_helpers)
            self.failUnless(r.endswith('rendered'))
        for h in cfg_helpers:
            self.failUnless(h in rendered_helpers.keys())

        rendered_helpers = slot_obj.getRenderedHelper(dummy1_inslot, (cfg_helpers[0],))
        self.assertEqual(len(rendered_helpers.keys()), 1)
        self.assertEqual(rendered_helpers.keys()[0], cfg_helpers[0])

        # test body rendering
        config = slot_obj._getConfigurationObject()
        setattr(slot_obj,
            "%s%s" % (config.getProperty('body_prefix'), dummy1_inslot.portal_type),
            lambda obj: 'body rendered correctly')
        self.assertEqual(slot_obj.getRenderedBody(dummy1_inslot),
            'body rendered correctly')

        # (2) update object
        setattr(dummy1, 'gaga', 1)
        dummy1_updated = getattr(aq_base(slot_obj), 'dummy1', None)
        self.failIfEqual(getattr(dummy1_updated, 'gaga'), 1)
        anno = dummy1_inslot.collection.getAnnotation()
        slot_obj.addOrUpdateObject('update', dummy1, dummy1_inslot, Names.UpToDate)
        dummy1_updated = getattr(aq_base(slot_obj), 'dummy1', None)
        self.assertEqual(getattr(dummy1_updated, 'gaga'), 1)
        self.assertEqual(slot_obj.getPos(dummy1_updated), dummy1_pos)
        self.assertEqual(slot_obj.getPos(dummy2_inslot), dummy2_pos)
        self.assertEqual(slot_obj.getStatus(dummy1_updated), Names.UpToDate)
        self.assertEqual(dummy1_updated.collection.getAnnotation(), anno)

        # (3) add different object with same id
        slot_obj.addOrUpdateObject('add', dummy22, None, Names.UpdateRequestPending)
        self.failUnless(getattr(aq_base(slot_obj), 'dummy22', None))

        # (4) test adding a collection to the collection
#       XXX: doesn't work with simulated collection !!! rewrite test
#        folder = self.root.portal.folder
#        self.failUnless(len(folder.objectIds()))
#        setattr(folder, 'meta_type', Names.collection_meta_type) # simulate collection
#        slot_obj.addOrUpdateObject('add', folder, None, Names.UpdateRequestPending)
#        folder_inslot = getattr(aq_base(slot_obj), 'folder', None)
#        self.failUnless(folder_inslot)
#        self.failIf(len(folder_inslot.objectIds()))

        # (5) test getting slot definition information
        self.assertEqual(slot_obj.getTitle(), self.slot_def_props['title'])
        self.assertEqual(slot_obj.getDescription(), self.slot_def_props['description'])
        self.assertEqual(slot_obj.getDefaultText(), self.slot_def_props['default_text'])

        # (6) test accepting and rejecting requests
        setattr(dummy1, 'gaga', 6.1)
        dummy1_inslot = getattr(aq_base(slot_obj), 'dummy1', None)
        dummy1_inslot.collection.status = Names.AddRequestPending
        self.failIfEqual(getattr(dummy1_inslot, 'gaga'), 6.1)
        slot_obj.acceptPendingRequest(dummy1_inslot)
        dummy1_updated = getattr(aq_base(slot_obj), 'dummy1', None)
        self.assertEqual(slot_obj.getStatus(dummy1_updated), Names.UpToDate)
        self.assertEqual(getattr(dummy1_updated, 'gaga'), 6.1)

        slot_obj.addOrUpdateObject('add', dummy5, None, Names.AddRequestPending)
        self.root.portal.manage_delObjects('dummy5')
        dummy5_inslot = getattr(aq_base(slot_obj), 'dummy5', None)
        self.failIfEqual(slot_obj.getStatus(dummy5_inslot), Names.DeleteNotificationPending)
        slot_obj.acceptPendingRequest(dummy5_inslot)
        dummy5_updated = getattr(aq_base(slot_obj), 'dummy5', None)
        self.assertEqual(slot_obj.getStatus(dummy5_updated), Names.IsLocal)

        setattr(dummy1, 'gaga', 6.2)
        dummy1_inslot = getattr(aq_base(slot_obj), 'dummy1', None)
        dummy1_inslot.collection.status = Names.AddRequestPending
        self.failIfEqual(getattr(dummy1_inslot, 'gaga'), 6.2)
        slot_obj.rejectPendingRequest(dummy1_inslot)
        self.assertEqual(getattr(aq_base(slot_obj), 'dummy1', None), None)

class SimpleCollectionTypeTests(SecurityTest):

    def setUp(self):
        SecurityTest.setUp(self)
        addCollectionType(self.root, 'test', 'test title')

    def test_factory(self):
        coll_obj = self.root.test
        self.assertEqual(coll_obj.getId(), 'test')
        self.assertEqual(coll_obj.Title(), 'test title')

    def test_addSlot(self):
        coll_obj = self.root.test
        addCollectionSlot(coll_obj, 'main', 'Main slot')
        slot_obj = coll_obj.main
        self.assertEqual(slot_obj.getId(), 'main')
        self.assertEqual(slot_obj.Title(), 'Main slot')


class ComplexCollectionTypeTests(SecurityTest):

    def setUp(self):
        SecurityTest.setUp(self)
        self.setUpToolInfrastructure()
        self.setUpContentInfrastructure()
        patchDummyDocument()

    def tearDown(self):
        unpatchDummyDocument()
        noSecurityManager()
        SecurityTest.tearDown(self)

    def setUpToolInfrastructure(self):
        # setup portal_catalog tool
        self.root._setObject('portal_catalog', CatalogTool())
        portal_catalog = self.root.portal_catalog
        portal_catalog.addIndex('puid', 'FieldIndex')
        portal_catalog.addColumn('puid')

        # setup portal_url tool
        self.root._setObject('portal_url', URLTool())

        # setup portal_collections tool with factory default configuration
        self.root._setObject(Names.tool_id, CollectionTool(Names.tool_id))
        portal_collections = getattr(self.root, Names.tool_id)

        # setup portal_types tool
        self.root._setObject('portal_types', TypesTool())
        portal_types = self.root.portal_types
        coll_config = getattr(portal_collections, Names.types_id)
        coll_id = default_types['_container'].keys()[0]
        portal_types.manage_addTypeInformation(
            FactoryTypeInformation.meta_type,
            id=coll_id,
            typeinfo_name=Types._buildTypeInfoName(coll_config),
            )
        portal_types._setObject('Dummy Content', DummyFTI) 
        portal_types._setObject('Dummy Folder', DummyFTI) 

    def setUpContentInfrastructure(self):
        # set up portal folder
        self.root._setObject('portal', PortalFolder('portal', ''))

        # setup dummy object id counter
        self.id_cnt = 0
        
        # add dummy content
        self.root.portal.invokeFactory('Dummy Content', 'dummy1')
        self.root.portal.invokeFactory('Dummy Content', 'dummy2')

        # add a collection of type coll_id
        coll_id = default_types['_container'].keys()[0]
        self.root.portal.invokeFactory(coll_id, 'collection')
        set_checkPermission(my_checkPermission)

        self.added_slot_id = 'Main'
        self.proposed_slot_id = 'Unassociated'

    def _addDummyObject(self, container):
        """Returns id and obj ref of newly created object
        """
        # build next id
        self.id_cnt += 1
        id = 'dummy_%s' % self.id_cnt

        obj = container.invokeFactory('Dummy Content', id)
        # CollectionTypes invokeFactory returns added object reference
        # whereas other types don't
        if obj is None:
            return id, getattr(container, id)
        return id, obj

    def setUpNewDummyObject(self):
        """Returns id and obj ref of newly created object
        """
        return self._addDummyObject(self.root.portal)

    def test_addOrUpdateObject(self):
        portal_collections = self.root.portal_collections
        collection = self.root.portal.collection

        ##### (1) add dummy object to collection conventionaly 
        #####     using invokeFactory
        # by calling getSlots all slots get added
        collection.getSlots()
        add_slot_ref = getattr(collection, self.added_slot_id)
        set_activePermission(Names.AddContentToCollections)
        dummy_id, dummy_obj = self._addDummyObject(collection)
        self.failUnless(getattr(aq_base(add_slot_ref), dummy_id, None))
        
        ##### (2) move object to another slot
        # by calling getSlots all slots get added
        collection.getSlots()
        add_slot_ref = getattr(collection, self.added_slot_id)
        propose_slot_ref = getattr(collection, self.proposed_slot_id)
        set_activePermission(Names.AddContentToCollections)
        dummy1_id, dummy1_obj = self._addDummyObject(collection)
        dummy2_id, dummy2_obj = self._addDummyObject(collection)
        self.failUnless(getattr(aq_base(add_slot_ref), dummy1_id, None))
        self.failUnless(getattr(aq_base(add_slot_ref), dummy2_id, None))
        dummies = ['%s/%s' % (self.added_slot_id,id) 
            for id in (dummy1_id,dummy2_id)]
        
        collection.moveObjects(dummies, self.proposed_slot_id)
        self.failIf(getattr(aq_base(add_slot_ref), dummy1_id, None))
        self.failIf(getattr(aq_base(add_slot_ref), dummy2_id, None))
        self.failUnless(getattr(aq_base(propose_slot_ref), dummy1_id, None))
        self.failUnless(getattr(aq_base(propose_slot_ref), dummy2_id, None))
        
        ##### (3) delete objects from different slots
        set_activePermission(Names.AddContentToCollections)
        dummy_main = []
        for i in range(3):
            dummy_main.append(self._addDummyObject(collection))
        ids_main = ['%s/%s' % (self.added_slot_id,d[0]) for d in dummy_main]
        dummy_unassoc = []
        for i in range(2):
            dummy_unassoc.append(self._addDummyObject(collection))
        ids_move = ['%s/%s' % (self.added_slot_id,d[0]) 
            for d in dummy_unassoc]
        ids_unassoc = ['%s/%s' % (self.proposed_slot_id,d[0]) 
            for d in dummy_unassoc]
        collection.moveObjects(ids_move, self.proposed_slot_id)
        dummy_id, dummy_obj = self._addDummyObject(collection)
        for d in dummy_main:
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))
        for d in dummy_unassoc:
            self.failUnless(getattr(aq_base(propose_slot_ref), d[0], None))
        # delete objects
        collection.manage_delObjects(ids=ids_main+ids_unassoc)
        # are the objects deleted
        for d in dummy_main:
            self.failIf(getattr(aq_base(add_slot_ref), d[0], None))
        for d in dummy_unassoc:
            self.failIf(getattr(aq_base(propose_slot_ref), d[0], None))
        # is dummy1 still there?
        self.failUnless(getattr(aq_base(add_slot_ref), dummy_id, None))
        
        ##### (4) rename objects of different slots
        set_activePermission(Names.AddContentToCollections)
        dummy_main = []
        for i in range(3):
            dummy_main.append(self._addDummyObject(collection))
        ids_main = ['%s/%s' % (self.added_slot_id,d[0]) for d in dummy_main]
        dummy_unassoc = []
        for i in range(2):
            dummy_unassoc.append(self._addDummyObject(collection))
        ids_move = ['%s/%s' % (self.added_slot_id,d[0]) 
            for d in dummy_unassoc]
        ids_unassoc = ['%s/%s' % (self.proposed_slot_id,d[0]) 
            for d in dummy_unassoc]
        collection.moveObjects(ids_move, self.proposed_slot_id)
        dummy_id, dummy_obj = self._addDummyObject(collection)
        for d in dummy_main:
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))
        for d in dummy_unassoc:
            self.failUnless(getattr(aq_base(propose_slot_ref), d[0], None))
        # rename objects
        ids = ids_main+ids_unassoc
        new_ids = ['%s_renamed' % d[0] for d in dummy_main+dummy_unassoc]
        collection.manage_renameObjects(ids=ids, new_ids=new_ids)
        # are the objects renamed
        for d in dummy_main:
            self.failIf(getattr(aq_base(add_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(add_slot_ref), '%s_renamed' % d[0], None))
        for d in dummy_unassoc:
            self.failIf(getattr(aq_base(propose_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(propose_slot_ref), '%s_renamed' % d[0], None))
        # is dummy1 still there?
        self.failUnless(getattr(aq_base(add_slot_ref), dummy_id, None))
        
        ##### (5) cut and paste objects from different slots in a 
        #####     collection to a folder
        set_activePermission(Names.AddContentToCollections)
        dummy_main = []
        for i in range(3):
            dummy_main.append(self._addDummyObject(collection))
        ids_main = ['%s/%s' % (self.added_slot_id,d[0]) for d in dummy_main]
        dummy_unassoc = []
        for i in range(2):
            dummy_unassoc.append(self._addDummyObject(collection))
        ids_move = ['%s/%s' % (self.added_slot_id,d[0]) 
            for d in dummy_unassoc]
        ids_unassoc = ['%s/%s' % (self.proposed_slot_id,d[0]) 
            for d in dummy_unassoc]
        collection.moveObjects(ids_move, self.proposed_slot_id)
        dummy_id, dummy_obj = self._addDummyObject(collection)
        for d in dummy_main:
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))
        for d in dummy_unassoc:
            self.failUnless(getattr(aq_base(propose_slot_ref), d[0], None))
        # cut and paste objects
        cp = collection.manage_cutObjects(ids=ids_main+ids_unassoc)
        self.root.portal.manage_pasteObjects(cb_copy_data=cp)
        # are the objects cut and pasted?
        for d in dummy_main:
            self.failIf(getattr(aq_base(add_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        for d in dummy_unassoc:
            self.failIf(getattr(aq_base(propose_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        # is dummy1 still there?
        self.failUnless(getattr(aq_base(add_slot_ref), dummy_id, None))
        
        ##### (6) cut and paste objects from a folder to a collection
        set_activePermission(Names.AddContentToCollections)
        dummy_objs = []
        for i in range(5):
            dummy_objs.append(self._addDummyObject(self.root.portal))
        ids = [d[0] for d in dummy_objs]
        for d in dummy_objs:
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        # cut and paste objects
        cp = self.root.portal.manage_cutObjects(ids=ids)
        collection.manage_pasteObjects(cb_copy_data=cp)
        # are the objects cut and pasted?
        for d in dummy_objs:
            self.failIf(getattr(aq_base(self.root.portal), d[0], None))
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))

        ##### (7) copy and paste objects from different slots in a 
        #####     collection to a folder
        set_activePermission(Names.AddContentToCollections)
        dummy_main = []
        for i in range(3):
            dummy_main.append(self._addDummyObject(collection))
        ids_main = ['%s/%s' % (self.added_slot_id,d[0]) for d in dummy_main]
        dummy_unassoc = []
        for i in range(2):
            dummy_unassoc.append(self._addDummyObject(collection))
        ids_move = ['%s/%s' % (self.added_slot_id,d[0]) 
            for d in dummy_unassoc]
        ids_unassoc = ['%s/%s' % (self.proposed_slot_id,d[0]) 
            for d in dummy_unassoc]
        collection.moveObjects(ids_move, self.proposed_slot_id)
        dummy_id, dummy_obj = self._addDummyObject(collection)
        for d in dummy_main:
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))
        for d in dummy_unassoc:
            self.failUnless(getattr(aq_base(propose_slot_ref), d[0], None))
        # copy and paste objects
        cp = collection.manage_copyObjects(ids=ids_main+ids_unassoc)
        self.root.portal.manage_pasteObjects(cb_copy_data=cp)
        # are the objects copy and pasted?
        for d in dummy_main:
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        for d in dummy_unassoc:
            self.failUnless(getattr(aq_base(propose_slot_ref), d[0], None))
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        # is dummy1 still there?
        self.failUnless(getattr(aq_base(add_slot_ref), dummy_id, None))

        ##### (8) copy and paste objects from a folder to a collection
        set_activePermission(Names.AddContentToCollections)
        dummy_objs = []
        for i in range(5):
            dummy_objs.append(self._addDummyObject(self.root.portal))
        ids = [d[0] for d in dummy_objs]
        for d in dummy_objs:
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
        # copy and paste objects
        cp = self.root.portal.manage_copyObjects(ids=ids)
        collection.manage_pasteObjects(cb_copy_data=cp)
        # are the objects copy and pasted?
        for d in dummy_objs:
            self.failUnless(getattr(aq_base(self.root.portal), d[0], None))
            self.failUnless(getattr(aq_base(add_slot_ref), d[0], None))


        # add object with AddContentToCollections permission
        dummy_id, dummy_obj = self.setUpNewDummyObject()
        set_activePermission(Names.AddContentToCollections)
        
        collection.addOrUpdateObject(dummy_obj)
        slot = getattr(collection, self.added_slot_id)
        
        dummy_inslot = getattr(slot, dummy_id, None)
        self.failUnless(dummy_inslot)
        self.assertEqual(portal_collections.PUID(dummy_obj), 
                         dummy_inslot.originals_puid)

        # update object with AddContentToCollections permission
        self.failIf(getattr(dummy_inslot, 'gaga', 0))
        dummy_obj.gaga = 1
        set_activePermission(Names.AddContentToCollections)
        collection.addOrUpdateObject(dummy_obj)
        dummy_updated = getattr(slot, dummy_id, None)
        self.assertEqual(dummy_updated.gaga, 1)

        # add object with ProposeContentToCollections permission
        dummy_id, dummy_obj = self.setUpNewDummyObject()
        set_activePermission(Names.ProposeContentToCollections)

        collection.addOrUpdateObject(dummy_obj)
        slot = getattr(collection, self.proposed_slot_id)
        
        dummy_inslot = getattr(slot, dummy_id, None)
        self.failUnless(dummy_inslot)
        self.assertEqual(portal_collections.PUID(dummy_obj), 
                         dummy_inslot.originals_puid)

        # update object with ProposeContentToCollections permission
        self.failIf(getattr(dummy_inslot, 'gaga', 0))
        dummy_obj.gaga = 1
        set_activePermission(Names.ProposeContentToCollections)
        collection.addOrUpdateObject(dummy_obj)
        dummy_updated = getattr(slot, dummy_id, None)
        self.assertEqual(dummy_updated.gaga, 1)
        
        ##################### more addAndUpdateObject tests here!!!

def test_suite():
    return TestSuite((
        makeSuite(CollectionToolConfigurationTests),
        makeSuite(CollectionToolTests),
        makeSuite(CollectionPuidToolTests),
        makeSuite(ObjectAnnotationTests),
        makeSuite(CollectionSlotTests),
        makeSuite(SimpleCollectionTypeTests),
#        makeSuite(ComplexCollectionTypeTests),
        ))

if __name__ == '__main__':
    main(defaultTest='test_suite')
